@@ -4,18 +4,6 @@
 // Note to the future.
 // TODO: explain why we need all these *_unprotected functions
 
-struct list_menu_settings {
-	size_t view_sel; // Index of the selected entry.
-	size_t view_min; // Index of the first visible entry.
-	size_t view_max; // Index of the last visible entry.
-	const struct wstring *entry_format;
-	bool (*enumerator)(size_t index); // Checks if index is valid.
-	const struct format_arg *(*get_args)(size_t index);
-	int (*paint_action)(size_t index);
-	void (*write_action)(size_t index, WINDOW *w);
-	bool (*unread_state)(size_t index);
-};
-
 size_t list_menu_height;
 size_t list_menu_width;
 
@@ -23,23 +11,15 @@ static WINDOW **windows = NULL;
 static size_t windows_count = 0;
 static size_t scrolloff;
 static size_t horizontal_shift = 0;
-
-static void list_menu_writer(size_t index, WINDOW *w);
-
-static struct list_menu_settings menus[MENUS_COUNT] = {
-	{0, 0, 0, NULL, &is_section_valid,   &get_section_args, &paint_section, &list_menu_writer,  &is_section_unread},
-	{0, 0, 0, NULL, &is_feed_valid,      &get_feed_args,    &paint_feed,    &list_menu_writer,  &is_feed_unread},
-	{0, 0, 0, NULL, &is_item_valid,      &get_item_args,    &paint_item,    &list_menu_writer,  &is_item_unread},
-	{0, 0, 0, NULL, &is_pager_pos_valid, NULL,              NULL,           &pager_menu_writer, NULL},
-};
-static struct list_menu_settings *menu = menus; // Selected menu.
-
 static struct wstring *list_fmtout = NULL;
 
-int8_t
-get_current_menu_type(void)
+static struct menu_state *menus = NULL;
+static struct menu_state *menu  = NULL;
+
+bool
+is_current_menu_a_pager(void)
 {
-	return menu - menus;
+	return menu != NULL && menu->enumerator == &is_pager_pos_valid ? true : false;
 }
 
 bool
@@ -64,7 +44,7 @@ adjust_list_menu(void)
 		}
 	}
 	windows_count = list_menu_height;
-	scrolloff = get_cfg_uint(CFG_SCROLLOFF);
+	scrolloff = get_cfg_uint(NULL, CFG_SCROLLOFF);
 	if (scrolloff > (list_menu_height / 2)) {
 		scrolloff = list_menu_height / 2;
 	}
@@ -87,15 +67,15 @@ free_list_menu(void)
 	free_wstring(list_fmtout);
 }
 
-static void
+void
 list_menu_writer(size_t index, WINDOW *w)
 {
-	if (menu->enumerator(index) == true) {
-		do_format(list_fmtout, menu->entry_format->ptr, menu->get_args(index));
+	if (menu->enumerator(menu, index) == true) {
+		do_format(list_fmtout, menu->entry_format->ptr, menu->get_args(menu, index));
 		if (list_fmtout->len > horizontal_shift) {
 			waddwstr(w, list_fmtout->ptr + horizontal_shift);
 		}
-		wbkgd(w, get_color_pair(menu->paint_action(index)) | (index == menu->view_sel ? A_REVERSE : 0));
+		wbkgd(w, get_cfg_color(NULL, menu->paint_action(menu, index)) | (index == menu->view_sel ? A_REVERSE : 0));
 	}
 }
 
@@ -147,40 +127,17 @@ redraw_list_menu_unprotected(void)
 		menu->view_max = menu->view_sel;
 		menu->view_min = menu->view_max - (list_menu_height - 1);
 	}
-	while (menu->view_max >= list_menu_height && menu->enumerator(menu->view_max) == false) {
+	while (menu->view_max >= list_menu_height && menu->enumerator(menu, menu->view_max) == false) {
 		menu->view_max -= 1;
 	}
 	menu->view_min = menu->view_max - (list_menu_height - 1);
 	expose_all_visible_entries_of_the_list_menu_unprotected();
 }
 
-const size_t *
-enter_list_menu(int8_t menu_index, config_entry_id format_id, bool do_reset)
-{
-	pthread_mutex_lock(&interface_lock);
-	struct list_menu_settings *prev_menu = menu;
-	menu = menus + menu_index;
-	if (do_reset == true) {
-		menu->view_sel = 0;
-		menu->view_min = 0;
-		menu->entry_format = get_cfg_wstring(format_id);
-	}
-	if (menu_index == SECTIONS_MENU) {
-		refresh_unread_items_count_of_all_sections();
-	}
-	horizontal_shift = 0;
-	if (do_reset == true || menu != prev_menu) {
-		status_clean_unprotected();
-		redraw_list_menu_unprotected();
-	}
-	pthread_mutex_unlock(&interface_lock);
-	return &(menu->view_sel);
-}
-
 void
 reset_list_menu_unprotected(void)
 {
-	if (menu->enumerator(menu->view_sel) == false) {
+	if (menu->enumerator(menu, menu->view_sel) == false) {
 		menu->view_sel = 0;
 		menu->view_min = 0;
 	}
@@ -188,27 +145,27 @@ reset_list_menu_unprotected(void)
 }
 
 static size_t
-obtain_list_entries_count_unprotected(struct list_menu_settings *m)
+obtain_list_entries_count_unprotected(struct menu_state *m)
 {
 	size_t i = 0;
-	while (m->enumerator(i) == true) {
+	while (m->enumerator(m, i) == true) {
 		i += 1;
 	}
 	return i;
 }
 
 static void
-change_list_view_unprotected(struct list_menu_settings *m, size_t new_sel)
+change_list_view_unprotected(struct menu_state *m, size_t new_sel)
 {
-	while (m->enumerator(new_sel) == false && new_sel > 0) {
+	while (m->enumerator(m, new_sel) == false && new_sel > 0) {
 		new_sel -= 1;
 	}
-	if (m->enumerator(new_sel) == false) {
+	if (m->enumerator(m, new_sel) == false) {
 		return;
 	}
 	if (new_sel + scrolloff > m->view_max) {
 		m->view_max = new_sel + scrolloff;
-		while (m->view_max >= list_menu_height && m->enumerator(m->view_max) == false) {
+		while (m->view_max >= list_menu_height && m->enumerator(m, m->view_max) == false) {
 			m->view_max -= 1;
 		}
 		m->view_min = m->view_max - (list_menu_height - 1);
@@ -236,11 +193,11 @@ change_list_view_unprotected(struct list_menu_settings *m, size_t new_sel)
 	} else if (new_sel != m->view_sel) {
 		if (m == menu) {
 			WINDOW *w = windows[m->view_sel - m->view_min];
-			wbkgd(w, get_color_pair(m->paint_action(m->view_sel)));
+			wbkgd(w, get_cfg_color(NULL, m->paint_action(m, m->view_sel)));
 			wnoutrefresh(w);
 			m->view_sel = new_sel;
 			w = windows[m->view_sel - m->view_min];
-			wbkgd(w, get_color_pair(m->paint_action(m->view_sel)) | A_REVERSE);
+			wbkgd(w, get_cfg_color(NULL, m->paint_action(m, m->view_sel)) | A_REVERSE);
 			wnoutrefresh(w);
 			doupdate();
 		} else {
@@ -250,18 +207,17 @@ change_list_view_unprotected(struct list_menu_settings *m, size_t new_sel)
 }
 
 bool
-handle_list_menu_control(uint8_t menu_id, input_cmd_id cmd, const struct wstring *arg)
+handle_list_menu_control(struct menu_state *m, input_id cmd, const struct wstring *arg)
 {
 	pthread_mutex_lock(&interface_lock);
-	struct list_menu_settings *m = &menus[menu_id];
 	if (cmd == INPUT_SELECT_NEXT || cmd == INPUT_JUMP_TO_NEXT) {
 		change_list_view_unprotected(m, m->view_sel + 1);
 	} else if (cmd == INPUT_SELECT_PREV || cmd == INPUT_JUMP_TO_PREV) {
 		change_list_view_unprotected(m, m->view_sel > 1 ? m->view_sel - 1 : 0);
 	} else if (cmd == INPUT_JUMP_TO_NEXT_UNREAD) {
 		for (size_t i = m->view_sel + 1, j = 0; j < 2; i = 0, ++j) {
-			while (m->enumerator(i) == true) {
-				if (m->unread_state(i) == true) {
+			while (m->enumerator(m, i) == true) {
+				if (m->unread_state(m, i) == true) {
 					change_list_view_unprotected(m, i);
 					j = 2;
 					break;
@@ -271,8 +227,8 @@ handle_list_menu_control(uint8_t menu_id, input_cmd_id cmd, const struct wstring
 		}
 	} else if (cmd == INPUT_JUMP_TO_PREV_UNREAD) {
 		for (size_t i = m->view_sel, j = 0; j < 2; ++j) {
-			while ((i > 0) && (m->enumerator(i - 1) == true)) {
-				if (m->unread_state(i - 1) == true) {
+			while (i > 0 && m->enumerator(m, i - 1) == true) {
+				if (m->unread_state(m, i - 1) == true) {
 					change_list_view_unprotected(m, i - 1);
 					j = 2;
 					break;
@@ -283,10 +239,10 @@ handle_list_menu_control(uint8_t menu_id, input_cmd_id cmd, const struct wstring
 				i = obtain_list_entries_count_unprotected(m);
 			}
 		}
-	} else if (cmd == INPUT_JUMP_TO_NEXT_IMPORTANT && menu_id == ITEMS_MENU) {
+	} else if (cmd == INPUT_JUMP_TO_NEXT_IMPORTANT && m->run == &items_menu_loop) {
 		for (size_t i = m->view_sel + 1, j = 0; j < 2; i = 0, ++j) {
-			while (m->enumerator(i) == true) {
-				if (important_item_condition(i) == true) {
+			while (m->enumerator(m, i) == true) {
+				if (important_item_condition(m, i) == true) {
 					change_list_view_unprotected(m, i);
 					j = 2;
 					break;
@@ -294,10 +250,10 @@ handle_list_menu_control(uint8_t menu_id, input_cmd_id cmd, const struct wstring
 				i += 1;
 			}
 		}
-	} else if (cmd == INPUT_JUMP_TO_PREV_IMPORTANT && menu_id == ITEMS_MENU) {
+	} else if (cmd == INPUT_JUMP_TO_PREV_IMPORTANT && m->run == &items_menu_loop) {
 		for (size_t i = m->view_sel, j = 0; j < 2; ++j) {
-			while (i > 0 && m->enumerator(i - 1) == true) {
-				if (important_item_condition(i - 1) == true) {
+			while (i > 0 && m->enumerator(m, i - 1) == true) {
+				if (important_item_condition(m, i - 1) == true) {
 					change_list_view_unprotected(m, i - 1);
 					j = 2;
 					break;
@@ -335,7 +291,7 @@ handle_list_menu_control(uint8_t menu_id, input_cmd_id cmd, const struct wstring
 		}
 	} else if (cmd == INPUT_SYSTEM_COMMAND) {
 		pthread_mutex_unlock(&interface_lock);
-		run_formatted_command(arg, m->get_args(m->view_sel));
+		run_formatted_command(arg, m->get_args(m, m->view_sel));
 		return true;
 	} else {
 		pthread_mutex_unlock(&interface_lock);
@@ -348,10 +304,10 @@ handle_list_menu_control(uint8_t menu_id, input_cmd_id cmd, const struct wstring
 static void
 change_pager_view_unprotected(size_t new_sel)
 {
-	while (menu->enumerator(new_sel) == false && new_sel > 0) {
+	while (menu->enumerator(menu, new_sel) == false && new_sel > 0) {
 		new_sel -= 1;
 	}
-	if (menu->enumerator(new_sel) == false) {
+	if (menu->enumerator(menu, new_sel) == false) {
 		return;
 	}
 	size_t entries_count = obtain_list_entries_count_unprotected(menu);
@@ -360,7 +316,7 @@ change_pager_view_unprotected(size_t new_sel)
 	}
 	if (new_sel != menu->view_min) {
 		menu->view_max = new_sel + (list_menu_height - 1);
-		while (menu->view_max >= list_menu_height && menu->enumerator(menu->view_max) == false) {
+		while (menu->view_max >= list_menu_height && menu->enumerator(menu, menu->view_max) == false) {
 			menu->view_max -= 1;
 		}
 		menu->view_min = menu->view_max - (list_menu_height - 1);
@@ -369,7 +325,7 @@ change_pager_view_unprotected(size_t new_sel)
 }
 
 bool
-handle_pager_menu_control(input_cmd_id cmd)
+handle_pager_menu_control(input_id cmd)
 {
 	pthread_mutex_lock(&interface_lock);
 	if (cmd == INPUT_SELECT_NEXT || cmd == INPUT_ENTER) {
@@ -391,3 +347,102 @@ handle_pager_menu_control(input_cmd_id cmd)
 	pthread_mutex_unlock(&interface_lock);
 	return true;
 }
+
+static inline void
+free_deleted_menus(void)
+{
+	while (menus != NULL && menus->is_deleted == true) {
+		struct menu_state *tmp = menus;
+		menus = menus->prev;
+		free_items_list(tmp->items);
+		free(tmp->feeds);
+		free(tmp);
+	}
+}
+
+void
+free_menus(void)
+{
+	while (menus != NULL) {
+		struct menu_state *tmp = menus;
+		menus = menus->prev;
+		free_items_list(tmp->items);
+		free(tmp->feeds);
+		free(tmp);
+	}
+}
+
+size_t
+get_menu_depth(void)
+{
+	size_t depth = 0;
+	for (struct menu_state *i = menus; i != NULL && i->is_deleted == false; i = i->prev) {
+		depth += 1;
+	}
+	return depth;
+}
+
+struct menu_state *
+setup_menu(struct menu_state *(*run)(struct menu_state *), struct feed_entry **feeds, size_t feeds_count, uint32_t flags)
+{
+	pthread_mutex_lock(&interface_lock);
+
+	// Get up-to-date information on unread items count
+	if (menus != NULL && menus->feeds_original != NULL && menus->feeds_count > 0) {
+		for (size_t i = 0; i < menus->feeds_count; ++i) {
+			int64_t unread_count = get_unread_items_count_of_the_feed(menus->feeds_original[i]->link);
+			if (unread_count >= 0) {
+				menus->feeds_original[i]->unread_count = unread_count;
+			}
+		}
+	}
+
+	struct menu_state *next_menu = NULL;
+	if (run != NULL) {
+		struct menu_state *new = calloc(1, sizeof(struct menu_state));
+		new->run            = run;
+		new->feeds_original = feeds;
+		new->feeds_count    = feeds_count;
+		new->flags          = flags;
+		new->prev           = menus;
+		if ((flags & MENU_SWALLOW) && menus != NULL) {
+			menus->is_deleted = true;
+		}
+		menus = new;
+		next_menu = menus;
+	} else {
+		for (struct menu_state *i = menus; i != NULL; i = i->prev) {
+			if (i->is_deleted == false) {
+				i->is_deleted = true;
+				break;
+			}
+		}
+		for (struct menu_state *i = menus; i != NULL; i = i->prev) {
+			if (i->is_deleted == false) {
+				next_menu = i;
+				next_menu->flags |= MENU_DISABLE_SETTINGS;
+				break;
+			}
+		}
+	}
+
+	pthread_mutex_unlock(&interface_lock);
+	return next_menu;
+}
+
+void
+start_menu(void)
+{
+	pthread_mutex_lock(&interface_lock);
+	free_deleted_menus();
+	menu = menus;
+	horizontal_shift = 0;
+	if (menu->is_initialized == false) {
+		menu->view_sel = 0;
+		menu->view_min = 0;
+		status_clean_unprotected();
+	}
+	redraw_list_menu_unprotected();
+	menu->is_initialized = true;
+	pthread_mutex_unlock(&interface_lock);
+}
