@@ -2,26 +2,18 @@
 #include <string.h>
 #include "newsraft.h"
 
-enum { // Even is ascending, odd is descending
-	ITEMS_SORT_BY_TIME_ASC = 0,
-	ITEMS_SORT_BY_TIME_DESC,
-	ITEMS_SORT_BY_UNREAD_ASC,
-	ITEMS_SORT_BY_UNREAD_DESC,
-	ITEMS_SORT_BY_ALPHABET_ASC,
-	ITEMS_SORT_BY_ALPHABET_DESC,
-	ITEMS_SORT_METHODS_COUNT,
-};
-
 static inline bool
-append_sorting_order_expression_to_query(struct string *query, int order)
+append_sorting_order_expression_to_query(struct string *q, int order)
 {
 	switch (order) {
-		case ITEMS_SORT_BY_TIME_ASC:      return catas(query, " ORDER BY MAX(publication_date, update_date) ASC, rowid ASC", 59);
-		case ITEMS_SORT_BY_TIME_DESC:     return catas(query, " ORDER BY MAX(publication_date, update_date) DESC, rowid DESC", 61);
-		case ITEMS_SORT_BY_UNREAD_ASC:    return catas(query, " ORDER BY unread ASC, MAX(publication_date, update_date) DESC, rowid DESC", 73);
-		case ITEMS_SORT_BY_UNREAD_DESC:   return catas(query, " ORDER BY unread DESC, MAX(publication_date, update_date) DESC, rowid DESC", 74);
-		case ITEMS_SORT_BY_ALPHABET_ASC:  return catas(query, " ORDER BY title ASC, rowid ASC", 30);
-		case ITEMS_SORT_BY_ALPHABET_DESC: return catas(query, " ORDER BY title DESC, rowid DESC", 32);
+		case SORT_BY_TIME_ASC:       return catas(q, " ORDER BY MAX(publication_date, update_date) ASC, rowid ASC", 59);
+		case SORT_BY_TIME_DESC:      return catas(q, " ORDER BY MAX(publication_date, update_date) DESC, rowid DESC", 61);
+		case SORT_BY_UNREAD_ASC:     return catas(q, " ORDER BY unread ASC, MAX(publication_date, update_date) DESC, rowid DESC", 73);
+		case SORT_BY_UNREAD_DESC:    return catas(q, " ORDER BY unread DESC, MAX(publication_date, update_date) DESC, rowid DESC", 74);
+		case SORT_BY_IMPORTANT_ASC:  return catas(q, " ORDER BY important ASC, MAX(publication_date, update_date) DESC, rowid DESC", 76);
+		case SORT_BY_IMPORTANT_DESC: return catas(q, " ORDER BY important DESC, MAX(publication_date, update_date) DESC, rowid DESC", 77);
+		case SORT_BY_ALPHABET_ASC:   return catas(q, " ORDER BY title ASC, rowid ASC", 30);
+		case SORT_BY_ALPHABET_DESC:  return catas(q, " ORDER BY title DESC, rowid DESC", 32);
 	}
 	return false;
 }
@@ -29,14 +21,16 @@ append_sorting_order_expression_to_query(struct string *query, int order)
 static inline struct string *
 generate_search_query_string(const struct items_list *items, const struct string *search_filter)
 {
-	struct string *q = crtas("SELECT rowid,feed_url,title,link,publication_date,update_date,unread,important FROM items WHERE (feed_url=?", 107);
+	struct string *q = crtas("SELECT rowid,feed_url,title,link,publication_date,update_date,unread,important FROM items WHERE feed_url IN (?", 110);
 	if (q == NULL) goto error;
 	for (size_t i = 1; i < items->feeds_count; ++i) {
-		if (catas(q, " OR feed_url=?", 14) == false) goto error;
+		if (catas(q, ",?", 2) == false) goto error;
 	}
 	if (catcs(q, ')') == false) goto error;
 	if (search_filter != NULL && search_filter->len > 0) {
-		if (catas(q, " AND (title LIKE '%' || ? || '%')", 33) == false) goto error;
+		if (catas(q, " AND ((title LIKE '%' || ? || '%') OR (content LIKE '%' || ? || '%'))", 69) == false) {
+			goto error;
+		}
 	}
 	if (append_sorting_order_expression_to_query(q, items->sorting) == false) goto error;
 	return q;
@@ -64,13 +58,13 @@ free_items_list(struct items_list *items)
 	}
 }
 
-static inline struct feed_entry *
+static inline struct feed_entry **
 find_feed_entry_by_url(struct feed_entry **feeds, size_t feeds_count, const char *feed_url)
 {
 	if (feed_url != NULL) {
 		for (size_t i = 0; i < feeds_count; ++i) {
 			if (strcmp(feed_url, feeds[i]->link->ptr) == 0) {
-				return feeds[i];
+				return feeds + i;
 			}
 		}
 	}
@@ -119,12 +113,14 @@ obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t ind
 
 		items->ptr[items->len].pub_date = sqlite3_column_int64(items->res, 4);
 		items->ptr[items->len].upd_date = sqlite3_column_int64(items->res, 5);
-		int64_t max_date = items->ptr[items->len].pub_date;
-		if (items->ptr[items->len].upd_date > max_date) {
-			max_date = items->ptr[items->len].upd_date;
+		if (items->ptr[items->len].pub_date <= 0 && items->ptr[items->len].upd_date > 0) {
+			items->ptr[items->len].pub_date = items->ptr[items->len].upd_date;
 		}
-		items->ptr[items->len].date_str = get_config_date_str(max_date, CFG_LIST_ENTRY_DATE_FORMAT);
-		items->ptr[items->len].pub_date_str = get_config_date_str(items->ptr[items->len].pub_date, CFG_LIST_ENTRY_DATE_FORMAT);
+		if (items->ptr[items->len].upd_date <= 0 && items->ptr[items->len].pub_date > 0) {
+			items->ptr[items->len].upd_date = items->ptr[items->len].pub_date;
+		}
+		items->ptr[items->len].date_str = get_cfg_date(NULL, CFG_LIST_ENTRY_DATE_FORMAT, items->ptr[items->len].upd_date);
+		items->ptr[items->len].pub_date_str = get_cfg_date(NULL, CFG_LIST_ENTRY_DATE_FORMAT, items->ptr[items->len].pub_date);
 
 		items->ptr[items->len].is_unread = sqlite3_column_int(items->res, 6); // unread
 		items->ptr[items->len].is_important = sqlite3_column_int(items->res, 7); // important
@@ -134,35 +130,34 @@ obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t ind
 }
 
 struct items_list *
-create_items_list(struct feed_entry **feeds, size_t feeds_count, int sorting, const struct string *search_filter)
+create_items_list(struct feed_entry **feeds, size_t feeds_count, int sorting, const struct wstring *search_filter)
 {
 	INFO("Generating items list.");
 	if (feeds_count == 0) {
 		return NULL;
 	}
-	struct items_list *items = malloc(sizeof(struct items_list));
+	struct items_list *items = calloc(1, sizeof(struct items_list));
 	if (items == NULL) {
 		return NULL;
 	}
 
 	items->sorting = sorting;
 	if (sorting < 0) {
-		items->sorting = ITEMS_SORT_BY_TIME_DESC;
-		const struct string *sort = get_cfg_string(CFG_MENU_ITEM_SORTING);
-		if      (strcmp(sort->ptr, "time-desc")     == 0) items->sorting = ITEMS_SORT_BY_TIME_DESC;
-		else if (strcmp(sort->ptr, "time-asc")      == 0) items->sorting = ITEMS_SORT_BY_TIME_ASC;
-		else if (strcmp(sort->ptr, "unread-desc")   == 0) items->sorting = ITEMS_SORT_BY_UNREAD_DESC;
-		else if (strcmp(sort->ptr, "unread-asc")    == 0) items->sorting = ITEMS_SORT_BY_UNREAD_ASC;
-		else if (strcmp(sort->ptr, "alphabet-desc") == 0) items->sorting = ITEMS_SORT_BY_ALPHABET_DESC;
-		else if (strcmp(sort->ptr, "alphabet-asc")  == 0) items->sorting = ITEMS_SORT_BY_ALPHABET_ASC;
+		items->sorting = SORT_BY_TIME_DESC;
+		const struct string *sort = get_cfg_string(NULL, CFG_MENU_ITEM_SORTING);
+		if      (strcmp(sort->ptr, "time-desc")      == 0) items->sorting = SORT_BY_TIME_DESC;
+		else if (strcmp(sort->ptr, "time-asc")       == 0) items->sorting = SORT_BY_TIME_ASC;
+		else if (strcmp(sort->ptr, "unread-desc")    == 0) items->sorting = SORT_BY_UNREAD_DESC;
+		else if (strcmp(sort->ptr, "unread-asc")     == 0) items->sorting = SORT_BY_UNREAD_ASC;
+		else if (strcmp(sort->ptr, "important-desc") == 0) items->sorting = SORT_BY_IMPORTANT_DESC;
+		else if (strcmp(sort->ptr, "important-asc")  == 0) items->sorting = SORT_BY_IMPORTANT_ASC;
+		else if (strcmp(sort->ptr, "alphabet-desc")  == 0) items->sorting = SORT_BY_ALPHABET_DESC;
+		else if (strcmp(sort->ptr, "alphabet-asc")   == 0) items->sorting = SORT_BY_ALPHABET_ASC;
 	}
 
-	items->ptr = NULL;
-	items->len = 0;
-	items->finished = false;
 	items->feeds = feeds;
 	items->feeds_count = feeds_count;
-	items->search_filter = search_filter == NULL ? NULL : crtss(search_filter);
+	items->search_filter = search_filter == NULL ? NULL : convert_wstring_to_string(search_filter);
 	items->query = generate_search_query_string(items, items->search_filter);
 	if (items->query == NULL) {
 		fail_status("Can't generate search query string!");
@@ -178,6 +173,7 @@ create_items_list(struct feed_entry **feeds, size_t feeds_count, int sorting, co
 	}
 	if (items->search_filter != NULL && items->search_filter->len > 0) {
 		db_bind_string(items->res, feeds_count + 1, items->search_filter);
+		db_bind_string(items->res, feeds_count + 2, items->search_filter);
 	}
 	obtain_items_at_least_up_to_the_given_index(items, 0);
 	if (items->len < 1) {
@@ -200,9 +196,11 @@ undo1:
 }
 
 bool
-replace_items_list_with_empty_one(struct items_list **items)
+recreate_items_list(struct items_list **items)
 {
-	struct items_list *new_items = create_items_list((*items)->feeds, (*items)->feeds_count, (*items)->sorting, (*items)->search_filter);
+	struct wstring *filter = (*items)->search_filter ? convert_string_to_wstring((*items)->search_filter) : NULL;
+	struct items_list *new_items = create_items_list((*items)->feeds, (*items)->feeds_count, (*items)->sorting, filter);
+	free_wstring(filter);
 	if (new_items == NULL) {
 		return false;
 	}
@@ -215,31 +213,39 @@ replace_items_list_with_empty_one(struct items_list **items)
 }
 
 void
-change_items_list_sorting(struct items_list **items, input_cmd_id cmd)
+change_items_list_sorting(struct items_list **items, input_id cmd)
 {
-	if (cmd == INPUT_SORT_BY_TIME) {
-		(*items)->sorting = (*items)->sorting == ITEMS_SORT_BY_TIME_DESC ? ITEMS_SORT_BY_TIME_ASC : ITEMS_SORT_BY_TIME_DESC;
-	} else if (cmd == INPUT_SORT_BY_UNREAD) {
-		(*items)->sorting = (*items)->sorting == ITEMS_SORT_BY_UNREAD_DESC ? ITEMS_SORT_BY_UNREAD_ASC : ITEMS_SORT_BY_UNREAD_DESC;
-	} else {
-		(*items)->sorting = (*items)->sorting == ITEMS_SORT_BY_ALPHABET_ASC ? ITEMS_SORT_BY_ALPHABET_DESC : ITEMS_SORT_BY_ALPHABET_ASC;
+	switch (cmd) {
+		case INPUT_SORT_BY_TIME:
+			(*items)->sorting = (*items)->sorting == SORT_BY_TIME_DESC ? SORT_BY_TIME_ASC : SORT_BY_TIME_DESC;
+			break;
+		case INPUT_SORT_BY_UNREAD:
+			(*items)->sorting = (*items)->sorting == SORT_BY_UNREAD_DESC ? SORT_BY_UNREAD_ASC : SORT_BY_UNREAD_DESC;
+			break;
+		case INPUT_SORT_BY_ALPHABET:
+			(*items)->sorting = (*items)->sorting == SORT_BY_ALPHABET_ASC ? SORT_BY_ALPHABET_DESC : SORT_BY_ALPHABET_ASC;
+			break;
+		case INPUT_SORT_BY_IMPORTANT:
+			(*items)->sorting = (*items)->sorting == SORT_BY_IMPORTANT_DESC ? SORT_BY_IMPORTANT_ASC : SORT_BY_IMPORTANT_DESC;
+			break;
 	}
-	replace_items_list_with_empty_one(items);
+	recreate_items_list(items);
 
 	const char *order = (*items)->sorting & 1 ? "descending" : "ascending";
 	switch ((*items)->sorting & ~1) {
-		case ITEMS_SORT_BY_TIME_ASC:     info_status("Sorted items by time (%s)", order); break;
-		case ITEMS_SORT_BY_UNREAD_ASC:   info_status("Sorted items by unread (%s)", order); break;
-		case ITEMS_SORT_BY_ALPHABET_ASC: info_status("Sorted items by alphabet (%s)", order); break;
+		case SORT_BY_TIME_ASC:      info_status("Sorted items by time (%s)", order); break;
+		case SORT_BY_UNREAD_ASC:    info_status("Sorted items by unread (%s)", order); break;
+		case SORT_BY_ALPHABET_ASC:  info_status("Sorted items by alphabet (%s)", order); break;
+		case SORT_BY_IMPORTANT_ASC: info_status("Sorted items by important (%s)", order); break;
 	}
 }
 
 void
-change_search_filter_of_items_list(struct items_list **items, const struct string *search_filter)
+change_search_filter_of_items_list(struct items_list **items, const struct wstring *search_filter)
 {
 	struct string *prev_search_filter = (*items)->search_filter;
-	(*items)->search_filter = crtss(search_filter);
-	if (replace_items_list_with_empty_one(items) == true) {
+	(*items)->search_filter = convert_wstring_to_string(search_filter);
+	if (recreate_items_list(items) == true) {
 		free_string(prev_search_filter);
 	} else {
 		free_string((*items)->search_filter);
