@@ -9,10 +9,10 @@
 #include "dates.c"
 #include "db.c"
 #include "db-items.c"
+#include "errors.c"
 #include "feeds.c"
 #include "feeds-parse.c"
 #include "interface.c"
-#include "interface-colors.c"
 #include "interface-input.c"
 #include "interface-list.c"
 #include "interface-list-pager.c"
@@ -26,7 +26,6 @@
 #include "items-metadata-persons.c"
 #include "items-pager.c"
 #include "load_config/config-auto.c"
-#include "load_config/config.c"
 #include "load_config/config-parse.c"
 #include "load_config/config-parse-colors.c"
 #include "load_config/config-parse-input.c"
@@ -50,17 +49,15 @@
 #include "update_feed/insert_feed/insert_feed.c"
 #include "update_feed/insert_feed/insert-feed-data.c"
 #include "update_feed/insert_feed/insert-item-data.c"
-#include "update_feed/parse_json/engage-json-parser.c"
-#include "update_feed/parse_xml/engage-xml-parser.c"
-#include "update_feed/parse_xml/xml-common.c"
-#include "update_feed/parse_xml/xml-handler-atom10.c"
-#include "update_feed/parse_xml/xml-handler-dublincore.c"
-#include "update_feed/parse_xml/xml-handler-georss.c"
-#include "update_feed/parse_xml/xml-handler-georss-gml.c"
-#include "update_feed/parse_xml/xml-handler-mediarss.c"
-#include "update_feed/parse_xml/xml-handler-rss.c"
-#include "update_feed/parse_xml/xml-handler-rsscontent.c"
-#include "update_feed/parse_xml/xml-handler-xhtml.c"
+#include "update_feed/parse_json/setup-json-parser.c"
+#include "update_feed/parse_xml/common.c"
+#include "update_feed/parse_xml/format-atom.c"
+#include "update_feed/parse_xml/format-dublincore.c"
+#include "update_feed/parse_xml/format-georss.c"
+#include "update_feed/parse_xml/format-mediarss.c"
+#include "update_feed/parse_xml/format-rss.c"
+#include "update_feed/parse_xml/gperf-data.c"
+#include "update_feed/parse_xml/setup-xml-parser.c"
 #include "update_feed/struct-item.c"
 #include "update_feed/update_feed.c"
 #include "wstring.c"
@@ -131,29 +128,33 @@ main(int argc, char **argv)
 		}
 	}
 
-	if (register_signal_handlers()         == false) { error = 6;  goto undo1; }
-	if (assign_default_binds()             == false) { error = 7;  goto undo1; }
-	if (parse_config_file()                == false) { error = 8;  goto undo2; }
-	if (load_config()                      == false) { error = 9;  goto undo3; }
-	if (db_init()                          == false) { error = 10; goto undo3; }
-	if (query_database_file_optimization() == false) { error = 11; goto undo4; }
-	if (parse_feeds_file()                 == false) { error = 12; goto undo4; }
-	if (curses_init()                      == false) { error = 13; goto undo5; }
-	if (adjust_list_menu()                 == false) { error = 14; goto undo6; }
-	if (status_recreate_unprotected()      == false) { error = 15; goto undo7; }
-	if (allocate_status_messages_buffer()  == false) { error = 16; goto undo8; }
-	if (counter_recreate_unprotected()     == false) { error = 17; goto undo8; }
-	if (curl_global_init(CURL_GLOBAL_DEFAULT)  != 0) { error = 18; goto undo9; }
-	refresh_unread_items_count_of_all_sections();
+	if (register_signal_handlers()         == false) { error = 6;  goto undo1;  }
+	if (assign_default_binds()             == false) { error = 7;  goto undo1;  }
+	if (curses_init()                      == false) { error = 13; goto undo2;  }
+	if (parse_config_file()                == false) { error = 8;  goto undo3;  }
+	if (load_config()                      == false) { error = 9;  goto undo4;  }
+	if (db_init()                          == false) { error = 10; goto undo4;  }
+	if (query_database_file_optimization() == false) { error = 11; goto undo5;  }
+	if (parse_feeds_file()                 == false) { error = 12; goto undo5;  }
+	if (adjust_list_menu()                 == false) { error = 14; goto undo6;  }
+	if (status_recreate_unprotected()      == false) { error = 15; goto undo7;  }
+	if (allocate_status_messages_buffer()  == false) { error = 16; goto undo8;  }
+	if (counter_recreate_unprotected()     == false) { error = 17; goto undo8;  }
+	if (curl_global_init(CURL_GLOBAL_DEFAULT)  != 0) { error = 18; goto undo9;  }
 	if (start_feed_updater()               == false) { error = 19; goto undo10; }
 
-	struct timespec check_period = {0, 100000000}; // 0.1 seconds
-	do {
-		enter_sections_menu_loop();
-		nanosleep(&check_period, NULL);
-	} while (!try_to_stop_feed_updater());
+	struct timespec idling = {0, 100000000}; // 0.1 seconds
+	struct menu_state *menu = setup_menu(&sections_menu_loop, NULL, 0, MENU_NO_FLAGS);
+	while (they_want_us_to_terminate == false) {
+		menu = menu->run(menu);
+		if (menu == NULL) {
+			if (try_to_stop_feed_updater()) break;
+			nanosleep(&idling, NULL); // Avoids CPU cycles waste while awaiting termination
+			menu = setup_menu(&sections_menu_loop, NULL, 0, MENU_DISABLE_SETTINGS);
+		}
+	}
 
-	clean_up_items_menu();
+	free_menus();
 undo10:
 	curl_global_cleanup();
 undo9:
@@ -163,17 +164,18 @@ undo8:
 undo7:
 	free_list_menu();
 undo6:
-	endwin();
-undo5:
 	free_sections();
-undo4:
+undo5:
 	db_stop();
-undo3:
+undo4:
 	free_config();
+undo3:
+	endwin();
 undo2:
-	free_binds();
+	free_binds(NULL);
 undo1:
 	log_stop(error);
 undo0:
+	flush_errors();
 	return error;
 }
