@@ -11,7 +11,7 @@
 #include <curses.h>
 
 #ifndef NEWSRAFT_VERSION
-#define NEWSRAFT_VERSION "0.23"
+#define NEWSRAFT_VERSION "0.24"
 #endif
 
 #define ISWHITESPACE(A) (((A)==' ')||((A)=='\n')||((A)=='\t')||((A)=='\v')||((A)=='\f')||((A)=='\r'))
@@ -26,9 +26,6 @@
 #define fail_status(...) status_write(CFG_COLOR_STATUS_FAIL, __VA_ARGS__)
 #define MIN(A, B) ((A) < (B) ? (A) : (B))
 
-#define NEWSRAFT_THREADS_COUNT_LIMIT 100
-
-#define NEWSRAFT_COLOR_PAIRS_COUNT 10
 typedef uint8_t config_entry_id;
 enum {
 	CFG_COLOR_STATUS_GOOD = 0,
@@ -41,6 +38,8 @@ enum {
 	CFG_COLOR_LIST_FEED_UNREAD,
 	CFG_COLOR_LIST_SECTION,
 	CFG_COLOR_LIST_SECTION_UNREAD,
+	CFG_RELOAD_PERIOD,
+	CFG_ITEM_LIMIT,
 	CFG_SCROLLOFF,
 	CFG_PAGER_WIDTH,
 	CFG_UPDATE_THREADS_COUNT,
@@ -55,6 +54,7 @@ enum {
 	CFG_USER_AGENT,
 	CFG_ITEM_CONTENT_FORMAT,
 	CFG_ITEM_CONTENT_DATE_FORMAT,
+	CFG_ITEM_CONTENT_LINK_FORMAT,
 	CFG_LIST_ENTRY_DATE_FORMAT,
 	CFG_OPEN_IN_BROWSER_COMMAND,
 	CFG_NOTIFICATION_COMMAND,
@@ -66,18 +66,19 @@ enum {
 	CFG_MENU_ITEM_SORTING,
 	CFG_SECTIONS_MENU_PARAMOUNT_EXPLORE,
 	CFG_FEEDS_MENU_PARAMOUNT_EXPLORE,
+	CFG_MARK_ITEM_UNREAD_ON_CHANGE,
 	CFG_MARK_ITEM_READ_ON_HOVER,
 	CFG_ANALYZE_DATABASE_ON_STARTUP,
 	CFG_CLEAN_DATABASE_ON_STARTUP,
 	CFG_RESPECT_TTL_ELEMENT,
 	CFG_RESPECT_EXPIRES_HEADER,
-	CFG_SEND_USER_AGENT_HEADER,
 	CFG_SEND_IF_NONE_MATCH_HEADER,
 	CFG_SEND_IF_MODIFIED_SINCE_HEADER,
+	CFG_PAGER_CENTERING,
 	CFG_ENTRIES_COUNT,
 };
 
-typedef uint8_t input_cmd_id;
+typedef uint8_t input_id;
 enum {
 	INPUT_SELECT_NEXT = 0,
 	INPUT_SELECT_PREV,
@@ -91,12 +92,14 @@ enum {
 	INPUT_JUMP_TO_PREV_UNREAD,
 	INPUT_JUMP_TO_NEXT_IMPORTANT,
 	INPUT_JUMP_TO_PREV_IMPORTANT,
+	INPUT_GOTO_FEED,
 	INPUT_SHIFT_WEST,
 	INPUT_SHIFT_EAST,
 	INPUT_SHIFT_RESET,
 	INPUT_SORT_BY_TIME,
 	INPUT_SORT_BY_UNREAD,
 	INPUT_SORT_BY_ALPHABET,
+	INPUT_SORT_BY_IMPORTANT,
 	INPUT_ENTER,
 	INPUT_RELOAD,
 	INPUT_RELOAD_ALL,
@@ -117,7 +120,6 @@ enum {
 	INPUT_SYSTEM_COMMAND,
 	INPUT_ERROR,
 	INPUT_APPLY_SEARCH_MODE_FILTER,
-	INPUT_ITEMS_MENU_WAS_NOT_CREATED,
 };
 
 typedef int8_t feeds_column_id;
@@ -156,13 +158,35 @@ enum {
 };
 
 enum {
-	SECTIONS_MENU,
+	SECTIONS_MENU = 0,
 	FEEDS_MENU,
 	ITEMS_MENU,
 	PAGER_MENU,
 	MENUS_COUNT,
 };
 
+enum {
+	MENU_NO_FLAGS         = 0,
+	MENU_IS_EXPLORE       = 1, // Tell items menu to use explore mode
+	MENU_USE_SEARCH       = 2, // Tell items menu to apply search
+	MENU_SWALLOW          = 4, // Replace current menu with this menu
+	MENU_DISABLE_SETTINGS = 8,
+};
+
+enum { // Even is ascending, odd is descending
+	SORT_BY_ORIGINAL_ASC = 0,
+	SORT_BY_ORIGINAL_DESC,
+	SORT_BY_TIME_ASC,
+	SORT_BY_TIME_DESC,
+	SORT_BY_UNREAD_ASC,
+	SORT_BY_UNREAD_DESC,
+	SORT_BY_ALPHABET_ASC,
+	SORT_BY_ALPHABET_DESC,
+	SORT_BY_IMPORTANT_ASC,
+	SORT_BY_IMPORTANT_DESC,
+	SORT_METHODS_COUNT,
+};
+
 typedef uint8_t render_block_format;
 enum {
 	TEXT_PLAIN,
@@ -181,6 +205,9 @@ enum {
 	FORMAT_ALL_END = 42, // Sum of all ending hints.
 };
 
+struct config_context;
+struct deserialize_stream;
+
 struct string {
 	char *ptr;
 	size_t len;
@@ -193,20 +220,32 @@ struct wstring {
 	size_t lim;
 };
 
+struct binding_action {
+	input_id cmd;
+	struct wstring *exec;
+};
+
+struct input_binding {
+	struct string *key;
+	struct binding_action *actions;
+	size_t actions_count;
+	struct input_binding *next;
+};
+
 struct feed_entry {
 	struct string *name;
 	struct string *link;
 	int64_t unread_count;
 	int64_t download_date;
-	int64_t update_period;
-	int64_t item_limit;
+	struct config_context *cfg;
+	struct input_binding *binds;
 	volatile bool *volatile did_update_just_finished;
 };
 
 struct item_entry {
 	struct string *title;
 	struct string *url;
-	struct feed_entry *feed;
+	struct feed_entry **feed;
 	int64_t rowid;
 	bool is_unread;
 	bool is_important;
@@ -224,7 +263,7 @@ struct items_list {
 	size_t len;
 	bool finished;
 	int sorting;
-	struct feed_entry **feeds; // Just a pointer to parent feeds.
+	struct feed_entry **feeds; // Just a pointer to parent feeds
 	size_t feeds_count;
 };
 
@@ -237,6 +276,28 @@ struct format_arg {
 	} value;
 };
 
+struct menu_state {
+	struct menu_state *(*run)(struct menu_state *); // Function used to start menu
+	struct feed_entry **feeds_original; // Remains unchanged to use original order
+	struct feed_entry **feeds;          // Virtual feeds with user sorting applied
+	size_t feeds_count;                 // Size of feeds_original and feeds arrays
+	struct items_list *items;
+	size_t items_age;                   // Refresh, if it doesn't match global age
+	uint32_t flags;
+	size_t view_sel;                    // Index of the selected entry
+	size_t view_min;                    // Index of the first visible entry
+	size_t view_max;                    // Index of the last visible entry
+	bool is_initialized;
+	bool is_deleted;
+	const struct wstring *entry_format;
+	bool (*enumerator)(struct menu_state *ctx, size_t index); // Checks if index is valid
+	const struct format_arg *(*get_args)(struct menu_state *ctx, size_t index);
+	int (*paint_action)(struct menu_state *ctx, size_t index);
+	void (*write_action)(size_t index, WINDOW *w);
+	bool (*unread_state)(struct menu_state *ctx, size_t index);
+	struct menu_state *prev;
+};
+
 struct render_block {
 	struct wstring *content;
 	render_block_format content_type;
@@ -278,73 +339,65 @@ struct links_list {
 	size_t len;
 };
 
-struct deserialize_stream;
-
 // See "sections.c" file for implementation.
 int64_t make_sure_section_exists(const struct string *section_name);
-bool copy_feed_to_section(const struct feed_entry *feed, int64_t section_index);
+struct feed_entry *copy_feed_to_section(const struct feed_entry *feed_data, int64_t section_index);
 void refresh_unread_items_count_of_all_sections(void);
 bool purge_abandoned_feeds(void);
-void enter_sections_menu_loop(void);
+struct menu_state *sections_menu_loop(struct menu_state *m);
 void free_sections(void);
 void process_auto_updating_feeds(void);
-bool is_section_valid(size_t index);
-const struct format_arg *get_section_args(size_t index);
-int paint_section(size_t index);
-bool is_section_unread(size_t index);
 void mark_feeds_read(struct feed_entry **feeds, size_t feeds_count, bool status);
+#ifdef TEST
+struct feed_entry **get_all_feeds(size_t *feeds_count);
+#endif
 
 // See "feeds-parse.c" file for implementation.
 bool parse_feeds_file(void);
 
 // See "feeds.c" file for implementation.
-bool is_feed_valid(size_t index);
-const struct format_arg *get_feed_args(size_t index);
-int paint_feed(size_t index);
-bool is_feed_unread(size_t index);
-input_cmd_id enter_feeds_menu_loop(struct feed_entry **base_feeds, size_t base_feeds_count, struct feed_entry **feeds_view);
+struct menu_state *feeds_menu_loop(struct menu_state *m);
 
 // See "interface-list.c" file for implementation.
-int8_t get_current_menu_type(void);
+bool is_current_menu_a_pager(void);
 bool adjust_list_menu(void);
 void free_list_menu(void);
+void list_menu_writer(size_t index, WINDOW *w);
 void expose_entry_of_the_list_menu(size_t index);
 void expose_all_visible_entries_of_the_list_menu(void);
 void redraw_list_menu_unprotected(void);
-const size_t *enter_list_menu(int8_t menu_index, config_entry_id format_id, bool do_reset);
 void reset_list_menu_unprotected(void);
-bool handle_list_menu_control(uint8_t menu_id, input_cmd_id cmd, const struct wstring *arg);
-bool handle_pager_menu_control(input_cmd_id cmd);
+bool handle_list_menu_control(struct menu_state *m, input_id cmd, const struct wstring *arg);
+bool handle_pager_menu_control(input_id cmd);
+void free_menus(void);
+size_t get_menu_depth(void);
+struct menu_state *setup_menu(struct menu_state *(*run)(struct menu_state *), struct feed_entry **feeds, size_t feeds_count, uint32_t flags);
+void start_menu(void);
 
 // See "interface-list-pager.c" file for implementation.
-bool is_pager_pos_valid(size_t index);
+bool is_pager_pos_valid(struct menu_state *ctx, size_t index);
 void pager_menu_writer(size_t index, WINDOW *w);
-bool start_pager_menu(struct render_blocks_list *new_blocks);
+bool start_pager_menu(struct config_context **new_ctx, struct render_blocks_list *new_blocks);
 bool refresh_pager_menu(void);
 
 // See "format.c" file for implementation.
 void do_format(struct wstring *dest, const wchar_t *fmt, const struct format_arg *args);
 
 // See "items.c" file for implementation.
-bool is_item_valid(size_t index);
-const struct format_arg *get_item_args(size_t index);
-int paint_item(size_t index);
-bool is_item_unread(size_t index);
-bool important_item_condition(size_t index);
-void clean_up_items_menu(void);
+bool important_item_condition(struct menu_state *ctx, size_t index);
 void tell_items_menu_to_regenerate(void);
-input_cmd_id enter_items_menu_loop(struct feed_entry **new_feeds, size_t new_feeds_count, bool is_explore_mode, const struct string *search_filter);
+struct menu_state *items_menu_loop(struct menu_state *dest);
 
 // See "items-list.c" file for implementation.
-struct items_list *create_items_list(struct feed_entry **feeds, size_t feeds_count, int sorting, const struct string *search_filter);
-bool replace_items_list_with_empty_one(struct items_list **items);
+struct items_list *create_items_list(struct feed_entry **feeds, size_t feeds_count, int sorting, const struct wstring *search_filter);
+bool recreate_items_list(struct items_list **items);
 void obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t index);
-void change_items_list_sorting(struct items_list **items, input_cmd_id cmd);
-void change_search_filter_of_items_list(struct items_list **items, const struct string *search_filter);
+void change_items_list_sorting(struct items_list **items, input_id cmd);
+void change_search_filter_of_items_list(struct items_list **items, const struct wstring *search_filter);
 void free_items_list(struct items_list *items);
 
 // See "items-pager.c" file for implementation.
-int enter_item_pager_view_loop(struct items_list *items, const size_t *view_sel);
+struct menu_state *item_pager_loop(struct menu_state *m);
 
 // Functions responsible for managing render blocks.
 // Render block is a piece of text in a single format. A list of render blocks
@@ -352,7 +405,7 @@ int enter_item_pager_view_loop(struct items_list *items, const size_t *view_sel)
 // and generates a single plain text buffer for a pager to display.
 // See "render-block.c" file for implementation.
 bool add_render_block(struct render_blocks_list *blocks, const char *content, size_t content_len, render_block_format content_type, bool needs_trimming);
-bool apply_links_render_blocks(struct render_blocks_list *blocks, const struct string *data);
+bool apply_links_render_blocks(struct render_blocks_list *blocks, const struct wstring *data);
 void free_render_blocks(struct render_blocks_list *blocks);
 
 // Here we extract links from texts of render_block entries into links_list and
@@ -364,7 +417,7 @@ void free_render_blocks(struct render_blocks_list *blocks);
 bool prepare_to_render_data(struct render_blocks_list *blocks, struct links_list *links);
 
 // See "render_data" directory for implementation.
-bool render_data(struct render_result *result, struct render_blocks_list *blocks, size_t content_width);
+bool render_data(struct config_context **ctx, struct render_result *result, struct render_blocks_list *blocks, size_t content_width);
 
 // See "items-metadata.c" file for implementation.
 bool generate_render_blocks_based_on_item_data(struct render_blocks_list *blocks, const struct item_entry *item, sqlite3_stmt *res);
@@ -376,7 +429,7 @@ bool get_largest_piece_from_item_attachments(const char *attachments, struct str
 // See "items-metadata-links.c" file for implementation.
 int64_t add_another_url_to_trim_links_list(struct links_list *links, const char *url, size_t url_len);
 bool populate_link_list_with_links_of_item(struct links_list *links, sqlite3_stmt *res);
-struct string *generate_link_list_string_for_pager(const struct links_list *links);
+struct wstring *generate_link_list_wstring_for_pager(struct config_context **ctx, const struct links_list *links);
 bool complete_urls_of_links(struct links_list *links);
 void free_links_list(const struct links_list *links);
 
@@ -393,7 +446,7 @@ const char *get_db_path(void);
 
 // See "dates.c" file for implementation.
 int64_t parse_date_rfc3339(const char *src);
-struct string *get_config_date_str(int64_t date, config_entry_id format_index);
+struct string *get_cfg_date(struct config_context **ctx, config_entry_id format_id, int64_t date);
 
 // See "db.c" file for implementation.
 bool db_init(void);
@@ -413,30 +466,26 @@ void db_set_download_date(const struct string *url, int64_t download_date);
 // See "db-items.c" file for implementation.
 sqlite3_stmt *db_find_item_by_rowid(int64_t rowid);
 bool db_mark_item_read(int64_t rowid, bool status);
-bool db_mark_item_important(int64_t rowid);
-bool db_mark_item_unimportant(int64_t rowid);
+bool db_mark_item_important(int64_t rowid, bool status);
 int64_t get_unread_items_count_of_the_feed(const struct string *url);
+int64_t get_items_count_of_feeds(struct feed_entry **feeds, size_t feeds_count);
 bool db_change_unread_status_of_all_items_in_feeds(struct feed_entry **feeds, size_t feeds_count, bool unread);
 
 // See "interface.c" file for implementation.
 bool curses_init(void);
-input_cmd_id resize_handler(void);
+input_id resize_handler(void);
 bool call_resize_handler_if_current_list_menu_size_is_different_from_actual(void);
+bool arent_we_colorful(void);
 
 // Functions related to window which reads user input and displays command
 // counter (prefix number before command).
 // See "interface-input.c" file for implementation.
 bool counter_recreate_unprotected(void);
 void tell_program_to_terminate_safely_and_quickly(int dummy);
-input_cmd_id get_input_command(uint32_t *count, const struct wstring **macro_ptr);
+input_id get_input(struct input_binding *ctx, uint32_t *count, const struct wstring **macro_ptr);
 void break_getting_input_command(void);
 void counter_delete(void);
 
-// Functions responsible for curses color pairs.
-// See "interface-colors.c" file for implementation.
-bool create_color_pairs(void);
-unsigned int get_color_pair(config_entry_id id);
-
 // Functions related to window which displays status messages.
 // See "interface-status.c" file for implementation.
 void update_status_window_content(void);
@@ -451,16 +500,16 @@ void status_delete(void);
 struct string *generate_string_with_status_messages_for_pager(void);
 
 // See "interface-status-pager.c" file for implementation.
-int enter_status_pager_view_loop(void);
+struct menu_state *status_pager_loop(struct menu_state *dest);
 
 // Functions responsible for managing of key bindings.
 // See "binds.c" file for implementation.
-input_cmd_id get_action_of_bind(int key_code, size_t action_index, const struct wstring **macro_ptr);
-ssize_t create_empty_bind_or_clean_existing(const char *key_name, size_t key_name_len);
-bool attach_cmd_action_to_bind(ssize_t bind_index, input_cmd_id cmd_action);
-bool attach_exec_action_to_bind(ssize_t bind_index, const char *exec, size_t exec_len);
+input_id get_action_of_bind(struct input_binding *ctx, const char *key, size_t action_index, const struct wstring **macro_ptr);
+struct input_binding *create_or_clean_bind(struct input_binding **target, const char *key);
+bool attach_action_to_bind(struct input_binding *bind, input_id action);
+bool attach_command_to_bind(struct input_binding *bind, const char *exec, size_t exec_len);
 bool assign_default_binds(void);
-void free_binds(void);
+void free_binds(struct input_binding *target);
 
 // Functions related to executing system commands.
 // See "commands.c" file for implementation.
@@ -476,17 +525,19 @@ bool cpyss(struct string **dest, const struct string *src);
 bool catas(struct string *dest, const char *src_ptr, size_t src_len);
 bool catss(struct string *dest, const struct string *src);
 bool catcs(struct string *dest, char c);
+bool make_string_fit_more(struct string **dest, size_t n);
 bool string_vprintf(struct string *dest, const char *format, va_list args);
 void empty_string(struct string *dest);
 void free_string(struct string *str);
 void trim_whitespace_from_string(struct string *str);
 struct wstring *convert_string_to_wstring(const struct string *src);
 struct wstring *convert_array_to_wstring(const char *src_ptr, size_t src_len);
+void remove_start_of_string(struct string *str, size_t size);
 void inlinefy_string(struct string *title);
 
 // See "string-serialize.c" file for implementation.
 bool serialize_caret(struct string **target);
-bool serialize_array(struct string **target, const char *key, size_t key_len, const char *value, size_t value_len);
+bool serialize_array(register struct string **target, register const char *key, register size_t key_len, register const char *value, register size_t value_len);
 bool serialize_string(struct string **target, const char *key, size_t key_len, const struct string *value);
 struct deserialize_stream *open_deserialize_stream(const char *serialized_data);
 const struct string *get_next_entry_from_deserialize_stream(struct deserialize_stream *stream);
@@ -508,22 +559,18 @@ struct string *convert_warray_to_string(const wchar_t *src_ptr, size_t src_len);
 // See "signal.c" file for implementation.
 bool register_signal_handlers(void);
 
-// Functions for opening and closing the log stream.
-// To write to the log stream use macros INFO, WARN or FAIL.
-// See "log.c" file for implementation.
-bool log_init(const char *path);
-void log_stop(int error_code);
-
 // Parse config file, fill out config_data structure, bind keys to actions.
 // See "load_config" directory for implementation.
+bool process_config_line(struct feed_entry *feed, const char *str, size_t len);
 bool parse_config_file(void);
 bool load_config(void);
-bool get_cfg_bool(config_entry_id i);
-size_t get_cfg_uint(config_entry_id i);
-unsigned int get_cfg_color(config_entry_id i, int *fg, int *bg);
-const struct string *get_cfg_string(config_entry_id i);
-const struct wstring *get_cfg_wstring(config_entry_id i);
+bool get_cfg_bool(struct config_context **ctx, config_entry_id id);
+size_t get_cfg_uint(struct config_context **ctx, config_entry_id id);
+unsigned int get_cfg_color(struct config_context **ctx, config_entry_id id);
+const struct string *get_cfg_string(struct config_context **ctx, config_entry_id id);
+const struct wstring *get_cfg_wstring(struct config_context **ctx, config_entry_id id);
 void free_config(void);
+void free_config_context(struct config_context *cfg);
 
 // Download, process and store new items of feed.
 // See "update_feed" directory for implementation.
@@ -531,11 +578,23 @@ void update_feeds(struct feed_entry **feeds, size_t feeds_count);
 bool start_feed_updater(void);
 bool try_to_stop_feed_updater(void);
 
+// Functions for opening and closing the log stream.
+// To write to the log stream use macros INFO, WARN or FAIL.
+// See "log.c" file for implementation.
+bool log_init(const char *path);
+void log_stop(int error_code);
+
+// Functions for buffering errors to prevent
+// ncurses calls from erasing printed text.
+// See "errors.c" file for implementation.
+void write_error(const char *format, ...);
+void flush_errors(void);
+
 extern volatile bool they_want_us_to_terminate;
 extern FILE *log_stream;
 extern size_t list_menu_height;
 extern size_t list_menu_width;
 extern bool search_mode_is_enabled;
-extern struct string *search_mode_text_input;
+extern struct wstring *search_mode_text_input;
 extern pthread_mutex_t interface_lock;
 #endif // NEWSRAFT_H
